
__pkgname__ = 'labtools-arna'
__version__ = '1.0.0'
__author__ = 'Luca Arnaboldi'
__license__ = 'GPLv3'
__summary__ = 'Tools for physics laborayory courses'
__url__ = 'https://github.com/arn4/LabTools'
